/** \file
	\brief armscope main source file
	\author Tomasz Ostrowski
*/

#include "board.h"
#include <stdlib.h>
#include <string.h>
//#include "dbgu.h"
#include "cdc_enumerate.h"
#include "board_armscope.h"
#include "delay.h"
#include "adc.h"
#include "timer.h"
#include "usb.h"
#include "buzzer.h"
#include "command.h"
#include "armscope_settings.h"

#if defined(__WinARM__) && !defined(UART_COM)
#warning "make sure syscalls.c is added to the source-file list (see makefile)"
#endif

static int iLastSentDataBuf;

static const char szId[] = "VDO/DMA v2b 20110623";
/** This is data added as header to each frame that isn't actual data but that is reply
	to some host request.
*/
static const unsigned char replyHeader[] = {0x00, 0xFF, 0x00, 0xFF, 0x00, 0xFF, 0x00, 0xFF, 
												0x00, 0xFF, 0x00, 0xFF, 0x00, 0xFF, 0x00, 0xFF, 
												0x00, 0xFF, 0x00, 0xFF, 0x00, 0xFF, 0x00, 0xFF };
												
int main ( void )
{
    char dataIn[21];
	struct S_ARMSCOPE_SETTINGS settings;
    unsigned int length=0;

	iLastSentDataBuf = iReadBufId;

    ADCInit();
    //* - Set the Clock frequency which supplies the timer, the mode
    AT91F_TC_Open ( AT91C_BASE_TC0,
                    (AT91C_TC_ASWTRG_CLEAR | AT91C_TC_ACPC_CLEAR | AT91C_TC_ACPA_SET | AT91C_TC_LDRA_RISING |
                     AT91C_TC_WAVE  | AT91C_TC_WAVESEL_UP_AUTO | TC_MCK_DIV ),
                    TIMER_ID);

    ConfigureIO();

    beep();
    beep();

    // Enable User Reset and set its minimal assertion to 960 us
    AT91C_BASE_RSTC->RSTC_RMR = AT91C_RSTC_URSTEN | (0x4<<8) | (unsigned int)(0xA5<<24);

    // Init USB device
    AT91F_USB_Open();

    // Set default GPIO values for analog front-end 
	//	clearX10();
    clearG2();
    clearG3();
    setG1();
    clearACDC();

	ADCStart();

    while (1)	
    {
        // Check enumeration
        if (pCDC.IsConfigured(&pCDC))
        {
            enum E_ACTION_REQUEST action = REQUEST_NONE;
			
			length += pCDC.Read(&pCDC, &dataIn[length], 3);		//non-blocking!
            if (length>=3)
            {                
                if (dataIn[2]==0x00)	//end of message detected - correct frame
                {
                    action = ExecuteCmd(dataIn, length, &settings);
                }
				length=0;
            }

			if (action == REQUEST_IDENTIFY)
			{
				// insert reply header
				pCDC.Write(&pCDC, (char*)replyHeader, sizeof(replyHeader));			
				pCDC.Write(&pCDC, szId, strlen(szId));			
				// pad reply to same size as regular data packet
				pCDC.Write(&pCDC, ADCbuf[iReadBufId], sizeof(ADCbuf[0]) - sizeof(replyHeader) - strlen(szId));
			}

			if (iReadBufId != iLastSentDataBuf)
			{
				pCDC.Write(&pCDC, ADCbuf[iReadBufId], sizeof(ADCbuf[0]));			
				//beep();
				iLastSentDataBuf = iReadBufId;
			}
			
        }	//if (pCDC.IsConfigured(&pCDC))
		else
		{
			iLastSentDataBuf = iReadBufId;		
		}
		
    }	//while (1)
}



